#include "main.h"

// clock init
void _init(void){ // called from the startup files
	//flash wait state 1
	REG_NVMCTRL_CTRLB |= (NVMCTRL_CTRLB_RWS(1));
	
	//XOSC (8MHz) enable
	SYSCTRL->XOSC.reg = (SYSCTRL_XOSC_ENABLE);
	while(!(SYSCTRL->PCLKSR.reg & SYSCTRL_PCLKSR_XOSCRDY));
	
	//clear DPLL reg
	SYSCTRL->DPLLCTRLA.reg = 0; 
	while(!(SYSCTRL->PCLKSR.reg & SYSCTRL_PCLKSR_B33SRDY));
	
	//DPLL to 96MHz
	SYSCTRL->DPLLRATIO.reg = SYSCTRL_DPLLRATIO_LDR(3000);
	SYSCTRL->DPLLCTRLB.reg = SYSCTRL_DPLLCTRLB_DIV(124) | SYSCTRL_DPLLCTRLB_REFCLK(SYSTEM_CLOCK_SOURCE_DPLL_REFERENCE_CLOCK_XOSC);
	SYSCTRL->DPLLCTRLA.reg = SYSCTRL_DPLLCTRLA_ENABLE;
	while(!(SYSCTRL->DPLLSTATUS.reg & (SYSCTRL_DPLLSTATUS_CLKRDY | SYSCTRL_DPLLSTATUS_LOCK)) == (SYSCTRL_DPLLSTATUS_CLKRDY | SYSCTRL_DPLLSTATUS_LOCK));
	
	//48 MHz on GCLK0 for main system (div 2)
	GCLK->GENDIV.reg = ((0 << GCLK_GENDIV_ID_Pos) | (2 << GCLK_GENDIV_DIV_Pos));
	GCLK->GENCTRL.reg = ((0 << GCLK_GENCTRL_ID_Pos) | (GCLK_SOURCE_FDPLL << GCLK_GENCTRL_SRC_Pos)| (GCLK_GENCTRL_GENEN));
	GCLK->CLKCTRL.reg = ((GCLK_CLKCTRL_GEN_GCLK0) | (GCLK_CLKCTRL_CLKEN)) ;	
	while(GCLK->STATUS.bit.SYNCBUSY);
	
	//96 MHz on GCLK1 for TCC0 High freqency PWM (div 1)
	GCLK->GENDIV.reg = ((1 << GCLK_GENDIV_ID_Pos) | (1 << GCLK_GENDIV_DIV_Pos));
	GCLK->GENCTRL.reg = ((1 << GCLK_GENCTRL_ID_Pos) | (GCLK_SOURCE_FDPLL << GCLK_GENCTRL_SRC_Pos)| (GCLK_GENCTRL_GENEN));
	GCLK->CLKCTRL.reg = ((GCLK_CLKCTRL_GEN_GCLK1) | (GCLK_CLKCTRL_CLKEN)) ;	
	while(GCLK->STATUS.bit.SYNCBUSY);
	
	//8 MHz on GCLK2 for slower periphery (div 12)
	GCLK->GENDIV.reg = ((2 << GCLK_GENDIV_ID_Pos) | (12 << GCLK_GENDIV_DIV_Pos));
	GCLK->GENCTRL.reg = ((2 << GCLK_GENCTRL_ID_Pos) | (GCLK_SOURCE_FDPLL << GCLK_GENCTRL_SRC_Pos)| (GCLK_GENCTRL_GENEN));
	GCLK->CLKCTRL.reg = ((GCLK_CLKCTRL_GEN_GCLK2) | (GCLK_CLKCTRL_CLKEN)) ;	
	while(GCLK->STATUS.bit.SYNCBUSY);
}

// 32-bit RTC for time functions
void init_RTC_timer(void){ // tiucks with 1S
	PM->APBCMASK.reg |= PM_APBAMASK_RTC;
	
	GCLK->CLKCTRL.reg = GCLK_CLKCTRL_ID(RTC_GCLK_ID) | GCLK_CLKCTRL_CLKEN | GCLK_CLKCTRL_GEN(2); // 8Mhz source
	
	RTC->MODE0.CTRL.reg &= ~(RTC_MODE0_CTRL_ENABLE);
	RTC->MODE0.CTRL.reg = RTC_MODE0_CTRL_SWRST;
	while (RTC->MODE0.CTRL.bit.SWRST);
	RTC->MODE0.CTRL.reg |= RTC_MODE0_CTRL_MODE_COUNT32 | RTC_MODE0_CTRL_PRESCALER_DIV8; // 1Mhz
	RTC->MODE0.CTRL.reg |= RTC_MODE0_CTRL_ENABLE;	
}

// 32-bit micros
uint32_t micros(void){
	return RTC->MODE0.COUNT.reg;
}

//32 bit delayMicroseconds
void delayMicroseconds(uint32_t uS){
	uint32_t now = micros();
	while((uint32_t)(micros()-now) < uS);
}


// External Interrupt Controller for PINA25 (EIC5) (SW0 on the SAMD10 Xplained)
void init_EIC_input(void){
	PM->APBCMASK.reg |= PM_APBAMASK_EIC;
	
	GCLK->CLKCTRL.reg = GCLK_CLKCTRL_ID(EIC_GCLK_ID) | GCLK_CLKCTRL_CLKEN | GCLK_CLKCTRL_GEN(2); // 8Mhz source
	
	EIC->CONFIG[0].reg |= EIC_CONFIG_SENSE5_FALL | EIC_CONFIG_FILTEN5; // falling edge and filter enabled
	
	EIC->CTRL.reg |= EIC_CTRL_ENABLE;
	
	PORT->Group[0].PMUX[25/2].bit.PMUXO = MUX_PA25A_EIC_EXTINT5  & 0xFFFF; 
	PORT->Group[0].OUTSET.reg = PORT_PA25; // pullup
	PORT->Group[0].PINCFG[25].reg |= PORT_PINCFG_PMUXEN | PORT_PINCFG_PULLEN;
	
	EIC->INTENSET.reg = EIC_INTENSET_EXTINT5; // enable interrupt
	NVIC_EnableIRQ(EIC_IRQn);
}

//EIC 5 interrupt
void EIC_Handler(void){
	static uint8_t buttonStat = 1;
	if (EIC->INTFLAG.reg & EIC_INTFLAG_EXTINT5){
		EIC->INTFLAG.reg = EIC_INTFLAG_EXTINT5;
		if(buttonStat == 0){
			buttonStat = 1;
			PORT->Group[0].PINCFG[9].reg |= PORT_PINCFG_PMUXEN; // switch PA09 from Output to TCC0 PWM
		}else{
			buttonStat = 0;
			PORT->Group[0].PINCFG[9].reg &= ~(PORT_PINCFG_PMUXEN);// switch PA09 from TCC0 PWM to Output
		}
	}
}


// TCC0 Timer init
void Init_TCC0(void){
	PM->APBCMASK.reg |= PM_APBCMASK_TCC0; // enable power
	
	GCLK->CLKCTRL.reg = GCLK_CLKCTRL_ID(TCC0_GCLK_ID) | GCLK_CLKCTRL_CLKEN | GCLK_CLKCTRL_GEN(1); // clock at 96Mhz
	
	TCC0->CTRLA.reg &= ~(TCC_CTRLA_ENABLE); 
	while(TCC0->SYNCBUSY.reg & TCC_SYNCBUSY_ENABLE);
	
	TCC0->WAVE.reg = TCC_WAVE_WAVEGEN_NPWM; 
	TCC0->PER.reg = 1000;  //96Khz PWM
	TCC0->CTRLA.reg = TCC_CTRLA_PRESCALER_DIV1 | TCC_CTRLA_ENABLE; // 96Mhz
	while(TCC0->SYNCBUSY.reg & TCC_SYNCBUSY_ENABLE);
	
	TCC0->CC[3].reg = 0;
	while(TCC0->SYNCBUSY.reg & TCC_SYNCBUSY_CC3);
	
	//pin A9 as PWM output of CC3
	PORT->Group[0].PMUX[9/2].bit.PMUXO = PINMUX_PA09E_TCC0_WO3 & 0xFFFF; 
	PORT->Group[0].PINCFG[9].reg |= PORT_PINCFG_PMUXEN;
}

// TCC0 CC3 Set PWM duty
void setDuty(uint16_t duty){
	TCC0->CC[3].reg = duty;
	while(TCC0->SYNCBUSY.reg & TCC_SYNCBUSY_CC3);
}


int main(void){
	
	init_RTC_timer();
	
	init_EIC_input();
	
	Init_TCC0();
	
	PORT->Group[0].DIRSET.reg = PORT_PA09;
	
	while (1){
		static uint16_t i;
		
		// fade
		if(i == 0){ // in 
			for(i = 0; i < 1000;i++){
				setDuty(i);
				delayMicroseconds(500); // wait 0.5ms
			}
		}else{ // out
			for(i = 1000; i != 0;i--){
				setDuty(i);
				delayMicroseconds(500);
			}
		}
		
		//blink
		PORT->Group[0].OUTTGL.reg = PORT_PA09;
	}
}